"""
raven.utils.compat
~~~~~~~~~~~~~~~~~~~~~~~~~~

:copyright: (c) 2010-2012 by the Sentry Team, see AUTHORS for more details.
:license: BSD, see LICENSE for more details.
"""

try:
    from urllib.error import HTTPError
except ImportError:
    from urllib2 import HTTPError  # NOQA


try:
    from urllib.request import Request, urlopen
except ImportError:
    from urllib2 import Request, urlopen  # NOQA


try:
    from urllib import quote as urllib_quote
except ImportError:
    from urllib.parse import quote as urllib_quote  # NOQA


try:
    from queue import Queue
except ImportError:
    from Queue import Queue  # NOQA


try:
    import urlparse as _urlparse
except ImportError:
    from urllib import parse as _urlparse  # NOQA


try:
    from unittest2 import TestCase
except ImportError:
    from unittest import TestCase  # NOQA



import urlparse

from raven.utils.wsgi import get_headers, get_environ
from werkzeug.exceptions import ClientDisconnected
from flask import current_app


def get_user_info(request):
    """
        Requires Flask-Login (https://pypi.python.org/pypi/Flask-Login/) to be installed
        and setup
    """
    try:
        from flask_login import current_user
    except ImportError:
        return None

    if not hasattr(current_app, 'login_manager'):
        return None

    if current_user.is_authenticated():
        user_info = {
            'is_authenticated': True,
            'is_anonymous': current_user.is_anonymous(),
            'id': current_user.get_id(),
        }

        if 'SENTRY_USER_ATTRS' in current_app.config:
            for attr in current_app.config['SENTRY_USER_ATTRS']:
                if hasattr(current_user, attr):
                    user_info[attr] = getattr(current_user, attr)
    else:
        user_info = {
            'is_authenticated': False,
            'is_anonymous': current_user.is_anonymous(),
        }

    return user_info


def get_data_from_request(request):
    urlparts = urlparse.urlsplit(request.url)

    try:
        formdata = request.form
    except ClientDisconnected:
        formdata = {}

    result = {}

    result['sentry.interfaces.User'] = get_user_info(request)

    result['sentry.interfaces.Http'] = {
        'url': '%s://%s%s' % (urlparts.scheme, urlparts.netloc, urlparts.path),
        'query_string': urlparts.query,
        'method': request.method,
        'data': formdata,
        'headers': dict(get_headers(request.environ)),
        'env': dict(get_environ(request.environ)),
    }

    return result
